"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RETRY_COUNT = void 0;
exports.aoSpawn = aoSpawn;
exports.aoSend = aoSend;
exports.aoDryRun = aoDryRun;
exports.aoMessageResult = aoMessageResult;
exports.aoMessageResults = aoMessageResults;
exports.fetchProcessSrc = fetchProcessSrc;
exports.aoCreateProcess = aoCreateProcess;
const aoconnect_1 = require("@permaweb/aoconnect");
const config_1 = require("../helpers/config");
const endpoints_1 = require("../helpers/endpoints");
const utils_1 = require("../helpers/utils");
const gql_1 = require("./gql");
exports.RETRY_COUNT = 200;
function aoSpawn(args) {
    return __awaiter(this, void 0, void 0, function* () {
        const processId = yield (0, aoconnect_1.spawn)({
            module: args.module,
            scheduler: args.scheduler,
            signer: (0, aoconnect_1.createDataItemSigner)(args.wallet),
            tags: args.tags,
            data: args.data,
        });
        return processId;
    });
}
function aoSend(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tags = [{ name: 'Action', value: args.action }];
            if (args.tags)
                tags.push(...args.tags);
            const data = args.useRawData ? args.data : JSON.stringify(args.data);
            const txId = yield (0, aoconnect_1.message)({
                process: args.processId,
                signer: (0, aoconnect_1.createDataItemSigner)(args.wallet),
                tags: tags,
                data: data,
            });
            return txId;
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
function aoDryRun(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tags = [{ name: 'Action', value: args.action }];
            if (args.tags)
                tags.push(...args.tags);
            let dataPayload;
            if (typeof args.data === 'object') {
                dataPayload = JSON.stringify(args.data || {});
            }
            else if (typeof args.data === 'string') {
                // Try to parse json and throw an error if it can't
                try {
                    JSON.parse(args.data);
                }
                catch (e) {
                    console.error(e);
                    throw new Error('Invalid JSON data');
                }
                dataPayload = args.data;
            }
            const response = yield (0, aoconnect_1.dryrun)({
                process: args.processId,
                tags: tags,
                data: dataPayload,
            });
            if (response.Messages && response.Messages.length) {
                if (response.Messages[0].Data) {
                    return JSON.parse(response.Messages[0].Data);
                }
                else {
                    if (response.Messages[0].Tags) {
                        return response.Messages[0].Tags.reduce((acc, item) => {
                            acc[item.name] = item.value;
                            return acc;
                        }, {});
                    }
                }
            }
        }
        catch (e) {
            console.error(e);
        }
    });
}
function aoMessageResult(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { Messages } = yield (0, aoconnect_1.result)({ message: args.messageId, process: args.processId });
            if (Messages && Messages.length) {
                const response = {};
                Messages.forEach((message) => {
                    const action = (0, utils_1.getTagValue)(message.Tags, 'Action') || args.action;
                    let responseData = null;
                    const messageData = message.Data;
                    if (messageData) {
                        try {
                            responseData = JSON.parse(messageData);
                        }
                        catch (_a) {
                            responseData = messageData;
                        }
                    }
                    const responseStatus = (0, utils_1.getTagValue)(message.Tags, 'Status');
                    const responseMessage = (0, utils_1.getTagValue)(message.Tags, 'Message');
                    response[action] = {
                        id: args.messageId,
                        status: responseStatus,
                        message: responseMessage,
                        data: responseData,
                    };
                });
                return response;
            }
            else
                return null;
        }
        catch (e) {
            console.error(e);
        }
    });
}
function aoMessageResults(args) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const tags = [{ name: 'Action', value: args.action }];
            if (args.tags)
                tags.push(...args.tags);
            yield (0, aoconnect_1.message)({
                process: args.processId,
                signer: (0, aoconnect_1.createDataItemSigner)(args.wallet),
                tags: tags,
                data: JSON.stringify(args.data),
            });
            yield new Promise((resolve) => setTimeout(resolve, 1000));
            const messageResults = yield (0, aoconnect_1.results)({
                process: args.processId,
                sort: 'DESC',
                limit: 100,
            });
            if (messageResults && messageResults.edges && messageResults.edges.length) {
                const response = {};
                for (const result of messageResults.edges) {
                    if (result.node && result.node.Messages && result.node.Messages.length) {
                        const resultSet = [args.action];
                        if (args.responses)
                            resultSet.push(...args.responses);
                        for (const message of result.node.Messages) {
                            const action = (0, utils_1.getTagValue)(message.Tags, 'Action');
                            if (action) {
                                let responseData = null;
                                const messageData = message.Data;
                                if (messageData) {
                                    try {
                                        responseData = JSON.parse(messageData);
                                    }
                                    catch (_a) {
                                        responseData = messageData;
                                    }
                                }
                                const responseStatus = (0, utils_1.getTagValue)(message.Tags, 'Status');
                                const responseMessage = (0, utils_1.getTagValue)(message.Tags, 'Message');
                                if (action === 'Action-Response') {
                                    const responseHandler = (0, utils_1.getTagValue)(message.Tags, 'Handler');
                                    if (args.handler && args.handler === responseHandler) {
                                        response[action] = {
                                            status: responseStatus,
                                            message: responseMessage,
                                            data: responseData,
                                        };
                                    }
                                }
                                else {
                                    if (resultSet.indexOf(action) !== -1) {
                                        response[action] = {
                                            status: responseStatus,
                                            message: responseMessage,
                                            data: responseData,
                                        };
                                    }
                                }
                                if (Object.keys(response).length === resultSet.length)
                                    break;
                            }
                        }
                    }
                }
                return response;
            }
            return null;
        }
        catch (e) {
            console.error(e);
        }
    });
}
function waitForProcess(processId, _setStatus) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        let retries = 0;
        while (retries < exports.RETRY_COUNT) {
            yield new Promise((r) => setTimeout(r, 2000));
            const gqlResponse = yield (0, gql_1.getGQLData)({
                gateway: config_1.GATEWAYS.goldsky,
                ids: [processId],
            });
            if ((_a = gqlResponse === null || gqlResponse === void 0 ? void 0 : gqlResponse.data) === null || _a === void 0 ? void 0 : _a.length) {
                const foundProcess = gqlResponse.data[0].node.id;
                console.log(`Fetched transaction: ${foundProcess}`);
                return foundProcess;
            }
            else {
                console.log(`Transaction not found: ${processId}`);
                retries++;
            }
        }
        throw new Error(`Process not found, please try again`);
    });
}
function fetchProcessSrc(txId) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const srcFetch = yield fetch((0, endpoints_1.getTxEndpoint)(txId));
            return yield srcFetch.text();
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
// TODO: Bootloader
function handleProcessEval(args) {
    return __awaiter(this, void 0, void 0, function* () {
        let src = null;
        if (args.evalSrc)
            src = args.evalSrc;
        else if (args.evalTxId)
            src = yield fetchProcessSrc(args.evalTxId);
        if (src) {
            try {
                const evalMessage = yield aoSend({
                    processId: args.processId,
                    wallet: args.wallet,
                    action: 'Eval',
                    data: src,
                    tags: args.evalTags || null,
                    useRawData: true,
                });
                console.log(`Eval: ${evalMessage}`);
                const evalResult = yield aoMessageResult({
                    processId: args.processId,
                    messageId: evalMessage,
                    action: 'Eval',
                });
                return evalResult;
            }
            catch (e) {
                throw new Error(e);
            }
        }
        return null;
    });
}
function aoCreateProcess(args, statusCB) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const spawnArgs = {
                module: args.module || config_1.AO.module,
                scheduler: args.scheduler || config_1.AO.scheduler,
                wallet: args.wallet,
            };
            if (args.spawnData)
                spawnArgs.data = args.spawnData;
            if (args.spawnTags)
                spawnArgs.tags = args.spawnTags;
            statusCB && statusCB(`Spawning process...`);
            const processId = yield aoSpawn(spawnArgs);
            statusCB && statusCB(`Retrieving process...`);
            yield waitForProcess(processId, statusCB);
            statusCB && statusCB(`Process retrieved!`);
            if (args.evalTxId || args.evalSrc) {
                statusCB && statusCB('Sending eval...');
                try {
                    const evalResult = yield handleProcessEval({
                        processId: processId,
                        evalTxId: args.evalTxId || null,
                        evalSrc: args.evalSrc || null,
                        evalTags: args.evalTags,
                        wallet: args.wallet,
                    });
                    if (evalResult && statusCB)
                        statusCB('Eval complete');
                }
                catch (e) {
                    throw new Error(e);
                }
            }
            return processId;
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
//# sourceMappingURL=ao.js.map