"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAtomicAsset = createAtomicAsset;
exports.getAtomicAsset = getAtomicAsset;
exports.getAtomicAssets = getAtomicAssets;
exports.buildAsset = buildAsset;
const ao_1 = require("../common/ao");
const gql_1 = require("../common/gql");
const config_1 = require("../helpers/config");
const utils_1 = require("../helpers/utils");
// TODO: Render-With
// TODO: Thumbnail
// TODO: License
function createAtomicAsset(args, wallet, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a, _b;
        if (!validateAssetCreateArgs(args))
            throw new Error('Invalid arguments passed for atomic asset creation');
        let processSrc = null;
        try {
            processSrc = yield (0, ao_1.fetchProcessSrc)(config_1.AO.src.asset);
            if (processSrc) {
                processSrc = processSrc.replaceAll(`'<NAME>'`, `[[${args.title}]]`);
                processSrc = processSrc.replaceAll('<CREATOR>', args.creator ? args.creator : wallet.address);
                processSrc = processSrc.replaceAll('<TICKER>', 'ATOMIC');
                processSrc = processSrc.replaceAll('<DENOMINATION>', args.denomination ? args.denomination.toString() : '1');
                processSrc = processSrc.replaceAll('<SUPPLY>', args.supply ? args.supply.toString() : '1');
                if (args.collectionId)
                    processSrc = processSrc.replaceAll('<COLLECTION>', args.collectionId);
                if (!args.transferable)
                    processSrc = processSrc.replace('Transferable = true', 'Transferable = false');
            }
        }
        catch (e) {
            throw new Error(e);
        }
        const tags = buildAssetTags(args);
        const data = (_b = (_a = config_1.CONTENT_TYPES[args.contentType]) === null || _a === void 0 ? void 0 : _a.serialize(args.data)) !== null && _b !== void 0 ? _b : args.data;
        try {
            const assetId = yield (0, ao_1.aoCreateProcess)({
                spawnData: data,
                spawnTags: tags,
                wallet: wallet,
                evalSrc: processSrc,
            }, (status) => callback(status));
            const initMessage = yield (0, ao_1.aoSend)({
                processId: assetId,
                wallet: wallet,
                action: 'Add-Upload-To-Zone'
            });
            console.log(`Init upload message: ${initMessage}`);
            return assetId;
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
function getAtomicAsset(id) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const gqlResponse = yield (0, gql_1.getGQLData)({
                gateway: config_1.GATEWAYS.goldsky,
                ids: [id],
                tagFilters: null,
                owners: null,
                cursor: null,
            });
            if (gqlResponse && gqlResponse.data.length) {
                const asset = buildAsset(gqlResponse.data[0]);
                let state = {
                    ticker: null,
                    denomination: null,
                    balances: null,
                    transferable: null
                };
                const processState = yield (0, ao_1.aoDryRun)({
                    processId: asset.id,
                    action: 'Info',
                });
                if (processState) {
                    if (processState.Name || processState.name) {
                        asset.title = processState.Name || processState.name;
                    }
                    if (processState.Ticker || processState.ticker)
                        state.ticker = processState.Ticker || processState.ticker;
                    if (processState.Denomination || processState.denomination)
                        state.denomination = processState.Denomination || processState.denomination;
                    if (processState.Logo || processState.logo)
                        asset.thumbnail = processState.Logo || processState.logo;
                    if (processState.Balances) {
                        state.balances = Object.fromEntries(Object.entries(processState.Balances).filter(([_, value]) => Number(value) !== 0));
                    }
                    if (processState.Transferable !== undefined) {
                        state.transferable = processState.Transferable;
                    }
                    else {
                        state.transferable = true;
                    }
                }
                if (!state.balances) {
                    try {
                        const processBalances = yield (0, ao_1.aoDryRun)({
                            processId: asset.id,
                            action: 'Balances',
                        });
                        if (processBalances)
                            state.balances = processBalances;
                    }
                    catch (e) {
                        console.error(e);
                    }
                }
                return Object.assign(Object.assign({}, asset), state);
            }
            return null;
        }
        catch (e) {
            throw new Error(e.message || 'Error fetching atomic asset');
        }
    });
}
function getAtomicAssets(args) {
    return __awaiter(this, void 0, void 0, function* () {
        var _a;
        try {
            const gqlResponse = yield (0, gql_1.getGQLData)({
                gateway: config_1.GATEWAYS.arweave,
                ids: (_a = args.ids) !== null && _a !== void 0 ? _a : null,
                tagFilters: null,
                owners: null,
                cursor: null,
            });
            if (gqlResponse && gqlResponse.data.length) {
                return gqlResponse.data.map((element) => buildAsset(element));
            }
            return null;
        }
        catch (e) {
            throw new Error(e);
        }
    });
}
function buildAsset(element) {
    const asset = {
        id: element.node.id,
        owner: element.node.owner.address,
        creator: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.creator),
        title: getTitle(element),
        description: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.description),
        type: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.type),
        topics: getTopics(element),
        implementation: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.implements),
        contentType: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.contentType),
        renderWith: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.renderWith),
        thumbnail: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.thumbnail),
        udl: getLicense(element),
        collectionId: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.collectionId),
        dateCreated: getDateCreated(element),
        blockHeight: element.node.block ? element.node.block.height : 0,
    };
    return asset;
}
function getTitle(element) {
    return (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.title) ||
        (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.name) ||
        (0, utils_1.formatAddress)(element.node.id, false);
}
function getTopics(element) {
    return element.node.tags
        .filter(tag => tag.name.includes(config_1.TAGS.keys.topic.toLowerCase()))
        .map(tag => tag.value);
}
function getDateCreated(element) {
    if (element.node.block) {
        return element.node.block.timestamp * 1000;
    }
    const dateCreatedTag = (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.dateCreated);
    if (dateCreatedTag) {
        return Number(dateCreatedTag);
    }
    return 0;
}
function getLicense(element) {
    const license = (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.license);
    if (license && license === config_1.LICENSES.udl.address) {
        return {
            access: { value: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.access) },
            derivations: { value: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.derivations) },
            commercialUse: { value: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.commericalUse) },
            dataModelTraining: { value: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.dataModelTraining) },
            paymentMode: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.paymentMode),
            paymentAddress: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.paymentAddress),
            currency: (0, utils_1.getTagValue)(element.node.tags, config_1.TAGS.keys.currency),
        };
    }
    return null;
}
function buildAssetTags(args) {
    const tags = [
        { name: config_1.TAGS.keys.title, value: args.title },
        { name: config_1.TAGS.keys.description, value: args.description },
        { name: config_1.TAGS.keys.type, value: args.type },
        { name: config_1.TAGS.keys.contentType, value: args.contentType },
        { name: config_1.TAGS.keys.implements, value: 'ANS-110' },
        { name: config_1.TAGS.keys.dateCreated, value: new Date().getTime().toString() },
    ];
    args.topics.forEach((topic) => tags.push({ name: config_1.TAGS.keys.topic, value: topic }));
    if (args.creator) {
        tags.push({ name: config_1.TAGS.keys.creator, value: args.creator });
    }
    if (args.collectionId) {
        tags.push({ name: config_1.TAGS.keys.collectionId, value: args.collectionId });
    }
    return tags;
}
function validateAssetCreateArgs(args) {
    if (typeof args !== 'object' || args === null)
        return false;
    const requiredFields = ['title', 'description', 'type', 'topics', 'contentType', 'data'];
    for (const field of requiredFields) {
        if (!(field in args))
            return false;
    }
    if (typeof args.title !== 'string' || args.title.trim() === '')
        return false;
    if (typeof args.description !== 'string')
        return false;
    if (typeof args.type !== 'string' || args.type.trim() === '')
        return false;
    if (!Array.isArray(args.topics) || args.topics.length === 0)
        return false;
    if (typeof args.contentType !== 'string' || args.contentType.trim() === '')
        return false;
    if (args.data === undefined || args.data === null)
        return false;
    if ('creator' in args && typeof args.creator !== 'string')
        return false;
    if ('collectionId' in args && typeof args.collectionId !== 'string')
        return false;
    if ('supply' in args && (typeof args.supply !== 'number' || args.supply <= 0))
        return false;
    if ('transferable' in args && typeof args.transferable !== 'boolean')
        return false;
    return true;
}
//# sourceMappingURL=assets.js.map